use context essentials2021

data RumorMill:
  | no-one
  | gossip(name :: String, next1 :: RumorMill, next2 :: RumorMill)
end


#|
fun rumor-mill-template(rm :: RumorMill) -> ...:
  doc: "Template for a function with a RumorMill as input"
  cases (RumorMill) rm:
    | no-one => ...
    | gossip(name, n1, n2) =>
      ... name
      ... rumor-mill-template(n1)
      ... rumor-mill-template(n2)
  end
where:
  rumor-mill-remplate(...) is ...
end
|#


# An example rumor mill for Ginny to hear about Harry's supposed hypogriff tattoo. This could be given as a single definition, but, as in the slides, I've given names to the parts to make it easier to type and to use sub-trees for testing.

GINNY-MILL = gossip("Ginny", no-one, no-one)
ROMILDA-MILL = gossip("Romilda", no-one, GINNY-MILL)
VINCENT-MILL = gossip("Vincent", no-one, no-one)
DRACO-MILL = gossip("Draco", ROMILDA-MILL, VINCENT-MILL)
CHO-MILL = gossip("Cho", no-one, no-one)
PANSY-MILL = gossip("Pansy", CHO-MILL, DRACO-MILL)


fun is-informed(rm :: RumorMill, possible-gossip :: String) -> Boolean:
  doc: "Determine whether `possible-gossip` is informed about a rumor that starts in `rm`"
  cases (RumorMill) rm:
    | no-one => false
    | gossip(name, next1, next2) => 
      (possible-gossip == name) or 
      is-informed(next1, possible-gossip) or
      is-informed(next2, possible-gossip)
  end
where:
  is-informed(no-one, "Cho") is false
  is-informed(ROMILDA-MILL, "Draco") is false
  is-informed(PANSY-MILL, "Ginny") is true
  is-informed(GINNY-MILL, "Ginny") is true
  # No one tells Dobby anything. :-(
  is-informed(PANSY-MILL, "Dobby") is false
end


fun rumor-delay(rm :: RumorMill) -> Number:
  doc: "Determine the maximum number of days for a rumor to propagate at a rate of 1 day to pass it on"
  cases (RumorMill) rm:
    | no-one => -1
    | gossip(name, next1, next2) =>
      1 + 
      num-max(
        rumor-delay(next1), 
        rumor-delay(next2))
  end
where:
  rumor-delay(no-one) is -1
  rumor-delay(GINNY-MILL) is 0
  rumor-delay(PANSY-MILL) is 3
end
